using System.Collections.Generic;

namespace Interfaces
{
    public interface IDbProvider
    {
        List<RaceSession> GetRaceSessions(bool getActive);

        bool CreateRaceSession(RaceSession session);

        bool StopRaceSession(RaceSession session);

        bool DeleteRaceSession(int raceSessionId);

        List<Pilot> GetPilotsList();

        Pilot GetPilot(int pilotId);

        bool CreatePilot(Pilot pilot);

        bool UpdatePilot(Pilot pilot);

        int GetLastScannedId();

        void StoreTransponderLog(SatelliteLog log);

        bool DeletePilot(int id);
        bool RemoveTransponderAssociation(int pilotId);
    }
}